/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright (c) 2011 - 2012 Oracle and/or its affiliates. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License.  You can
 * obtain a copy of the License at
 * http://glassfish.java.net/public/CDDL+GPL_1_1.html
 * or packager/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at packager/legal/LICENSE.txt.
 *
 * GPL Classpath Exception:
 * Oracle designates this particular file as subject to the "Classpath"
 * exception as provided by Oracle in the GPL Version 2 section of the License
 * file that accompanied this code.
 *
 * Modifications:
 * If applicable, add the following below the License Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyright [year] [name of copyright owner]"
 *
 * Contributor(s):
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */

package org.glassfish.websocket.platform;

import org.glassfish.websocket.api.Peer;
import org.glassfish.websocket.api.Conversation;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
/**
 *
 * @author dannycoward
 */
public class WebSocketConversationImpl extends WithProperties implements Conversation<Peer> {
    private WebSocketWrapper wsw;
    private long conversationID;
    static long count = 0;
    private HttpSession httpSession;

    WebSocketConversationImpl(WebSocketWrapper wsw) {
        this.wsw = wsw;
        this.conversationID = count;
        count++;
    }

    void setHttpSession(HttpSession httpSession) {
        this.httpSession = httpSession;
    }

    public HttpSession getHttpSession() {
        return this.httpSession;
    }

    public String getSubprotocol() {
        return "not implemented";
    }

    public Peer getPeer() {
        return wsw;
    }

     /** Return a unique ID for this session.*/
    public Long getConversationID() {
        return new Long(count);
    }

    public boolean isActive() {
        return wsw.isConnected();
    }

    public Date getActivationTime() {
        return this.wsw.activationTime;
    }

    /** Closes the underlying connection this session is based upon.*/
    public void deactivate(int code, String reason) throws IOException {
        this.wsw.doClose(code, reason);
    }

    public String toString() {
        return "Session("+conversationID+", "+this.isActive()+")";
    }
}
